/**
 * @file
 * Statistics module
 *
 */

/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

#include "lwip/opt.h"
#include "lwip/tcp.h"

#if LWIP_STATS /* don't build if not configured for use in lwipopts.h */

#include "lwip/def.h"
#include "lwip/stats.h"
#include "lwip/mem.h"

#include <string.h>
#include <stdio.h>


// REM AZ Erweiterung
#ifdef	LWIP_STATS_PRINT

#ifdef	LWIP_PLATFORM_DIAG
#undef	LWIP_PLATFORM_DIAG
#endif

#define LWIP_PLATFORM_DIAG(a)	LWIP_STATS_PRINT(a)

#endif

extern struct tcp_pcb ** const tcp_pcb_lists[];		// REM AZ Erweiterung


struct stats_ lwip_stats;

void stats_init(void)
{
#ifdef LWIP_DEBUG
#if MEMP_STATS
  const char * memp_names[] = {
#define LWIP_MEMPOOL(name,num,size,desc) desc,
#include "lwip/memp_std.h"
  };
  int i;
  for (i = 0; i < MEMP_MAX; i++) {
    lwip_stats.memp[i].name = memp_names[i];
  }
#endif /* MEMP_STATS */
#if MEM_STATS
  lwip_stats.mem.name = "MEM";
#endif /* MEM_STATS */
#endif /* LWIP_DEBUG */
}

#if LWIP_STATS_DISPLAY
void
stats_display_proto(struct stats_proto *proto, const char *name LWIP_STATS_PARAMS_EX)
{
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_INIT"%s"                          LWIP_STATS_PARAMS_MIDD, name            LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "xmit:       %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->xmit     LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "recv:       %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->recv     LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "fw:         %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->fw       LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "drop:       %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->drop     LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "chkerr:     %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->chkerr   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "lenerr:     %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->lenerr   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "memerr:     %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->memerr   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "rterr:      %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->rterr    LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "proterr:    %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->proterr  LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "opterr:     %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->opterr   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "err:        %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->err      LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "cachehit:   %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, proto->cachehit LWIP_STATS_PARAMS_POST));
}

#if IGMP_STATS
void
stats_display_igmp(struct stats_igmp *igmp LWIP_STATS_PARAMS_EX)
{
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_INIT"IGMP"                        LWIP_STATS_PARAMS_MIDD                   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "xmit:       %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->xmit       LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "recv:       %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->recv       LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "drop:       %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->drop       LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "chkerr:     %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->chkerr     LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "lenerr:     %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->lenerr     LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "memerr:     %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->memerr     LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "proterr:    %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->proterr    LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "rx_v1:      %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->rx_v1      LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "rx_group:   %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->rx_group   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "rx_general: %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->rx_general LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "rx_report:  %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->rx_report  LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "tx_join:    %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->tx_join    LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "tx_leave:   %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->tx_leave   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "tx_report:  %"STAT_COUNTER_F LWIP_STATS_PARAMS_MIDD, igmp->tx_report  LWIP_STATS_PARAMS_POST));
}
#endif /* IGMP_STATS */

#if MEM_STATS || MEMP_STATS
void
stats_display_mem(struct stats_mem *mem, const char *name LWIP_STATS_PARAMS_EX)
{
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_INIT"MEM %s"             LWIP_STATS_PARAMS_MIDD, name              LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "avail:      %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)mem->avail LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "used:       %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)mem->used  LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "max:        %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)mem->max   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "err:        %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)mem->err   LWIP_STATS_PARAMS_POST));
}

#if MEMP_STATS
void
stats_display_memp(struct stats_mem *mem, int index LWIP_STATS_PARAMS_EX)
{
  char * memp_names[] = {
#define LWIP_MEMPOOL(name,num,size,desc) desc,
#include "lwip/memp_std.h"
  };
  if(index < MEMP_MAX) {
    stats_display_mem(mem, memp_names[index] LWIP_STATS_PARAMS_CL);
  }
}
#endif /* MEMP_STATS */
#endif /* MEM_STATS || MEMP_STATS */

#if SYS_STATS
void
stats_display_sys(struct stats_sys *sys LWIP_STATS_PARAMS_EX)
{
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_INIT"SYS"                LWIP_STATS_PARAMS_MIDD                         LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "sem.used:   %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)sys->sem.used   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "sem.max:    %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)sys->sem.max    LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "sem.err:    %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)sys->sem.err    LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "mutex.used: %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)sys->mutex.used LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "mutex.max:  %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)sys->mutex.max  LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "mutex.err:  %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)sys->mutex.err  LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "mbox.used:  %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)sys->mbox.used  LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "mbox.max:   %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)sys->mbox.max   LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "mbox.err:   %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)sys->mbox.err   LWIP_STATS_PARAMS_POST));
}
#endif /* SYS_STATS */

#if PIPE_STATS
void
stats_display_pipe(struct stats_pipe *pipe LWIP_STATS_PARAMS_EX)
{
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_INIT"PIPE"               LWIP_STATS_PARAMS_MIDD                         LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "input:      %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)pipe->input	    LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "inperr:     %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)pipe->inperr	LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "output:     %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)pipe->output	LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "outerr:     %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)pipe->outerr    LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "mboxnow:    %"S32_F LWIP_STATS_PARAMS_MIDD, (s32_t)pipe->mboxnow	LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "mboxmax:    %"S32_F LWIP_STATS_PARAMS_MIDD, (s32_t)pipe->mboxmax	LWIP_STATS_PARAMS_POST));
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "mboxerr:    %"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)pipe->mboxerr   LWIP_STATS_PARAMS_POST));
}
#endif /* CONN_STATS */

#if OPEN_STATS
void
stats_display_open(struct stats_open *open LWIP_STATS_PARAMS_EX)
{
  int i;	
  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_INIT"TCP_OPEN" LWIP_STATS_PARAMS_MIDD LWIP_STATS_PARAMS_POST));

  for (i = 0; i < OPEN_STATS_MAX; i++) {
	if(!open->port[i])continue;
    LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "port %5"U32_F": %"U32_F "/%"U32_F LWIP_STATS_PARAMS_MIDD, (u32_t)open->port[i],(u32_t)open->open[i],(u32_t)open->close[i] LWIP_STATS_PARAMS_POST)); 
    }
}

void
stats_open_change(struct stats_open *open,unsigned short port,int mode)
{
  int i,min,pos,val;	
		    
  for (i = 0; i < OPEN_STATS_MAX; i++) {
    if(open->port[i] != port) {
	  if(open->port[i] != 0) {
	    continue;
		}
 	  open->port[i] = port;
	  }

	if(mode)
	  open->open[i]++;
	else
	  open->close[i]++; 

	return;
	}

  min = open->open[i]+open->close[i];
  pos = 0;

  for (i = 1; i < OPEN_STATS_MAX; i++) {
	val = open->open[i]+open->close[i];
	if(val >= min)
	  continue;
	min = val;
	pos = i;
	}

  open->port [i] = port;
  open->open [i] = 0;
  open->close[i] = 0;

  if(mode)
    open->open[pos]++;
  else
    open->close[pos]++; 
}
#endif /* OPEN_STATS */

#if CONN_STATS
void
stats_display_conn(struct tcp_pcb ** const *pcb_lists,int count LWIP_STATS_PARAMS_EX)
{
  struct tcp_pcb *cpcb;
  char remote[64];
  char local [64];
  int  i;

  SYS_ARCH_DECL_PROTECT(old_level);

  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_INIT"CONN" LWIP_STATS_PARAMS_MIDD LWIP_STATS_PARAMS_POST));

  SYS_ARCH_PROTECT(old_level);

  for (i = 0; i < count; i++) {
    for(cpcb = *pcb_lists[i]; cpcb != NULL; cpcb = cpcb->next) {
	  if(cpcb->state==CLOSED)continue;
	  sprintf(remote,"%u.%u.%u.%u",(unsigned char)cpcb->remote_ip.addr,(unsigned char)(cpcb->remote_ip.addr>>8),(unsigned char)(cpcb->remote_ip.addr>>16),(unsigned char)(cpcb->remote_ip.addr>>24));
	  sprintf(local ,"%u.%u.%u.%u",(unsigned char)cpcb->local_ip.addr,(unsigned char)(cpcb->local_ip.addr>>8),(unsigned char)(cpcb->local_ip.addr>>16),(unsigned char)(cpcb->local_ip.addr>>24));
	  if(cpcb->state==LISTEN)remote[0]=0;
	  LWIP_PLATFORM_DIAG((LWIP_STATS_PARAMS_PRE "%-16s %-16s  %5u %5u  %s"LWIP_STATS_PARAMS_MIDD,local,remote,cpcb->local_port,cpcb->remote_port,tcp_debug_state_str(cpcb->state) LWIP_STATS_PARAMS_POST));
    }
  }

  SYS_ARCH_UNPROTECT(old_level);
}
#endif /* CONN_STATS */

void
stats_display(LWIP_STATS_PARAMS)
{
  s16_t i;

  PIPE_STATS_DISPLAY();
  LINK_STATS_DISPLAY();
  ETHARP_STATS_DISPLAY();
  IPFRAG_STATS_DISPLAY();
  IP_STATS_DISPLAY();
  IGMP_STATS_DISPLAY();
  ICMP_STATS_DISPLAY();
  UDP_STATS_DISPLAY();
  TCP_STATS_DISPLAY();
  MEM_STATS_DISPLAY();
  for (i = 0; i < MEMP_MAX; i++) {
    MEMP_STATS_DISPLAY(i);
  }
  SYS_STATS_DISPLAY();
  CONN_STATS_DISPLAY();
}
#endif /* LWIP_STATS_DISPLAY */

#endif /* LWIP_STATS */

